#!/usr/bin/perl

#Assignment Two: Perl Moderately Complex Assignment - Coaster Counter(for rollercoaster enthusiasts)
#Author: Nicholas Piccillo
#
#This program will track rollercoasters ridden and: 
#provide a summary of coasters by park name
#provide a total number of coasters ridden
#countdown to coaster count milestones (50, 100, 200, 300, 400, 500, 1000, 1000+)
#
#Coaster information will be saved to and read from a file in json format
#
#This program does not accept command line arguments
#
#Note: 	with more time I would add multiple users, command line arguments (for add and user name),
#	    and checking for duplicate coasters

use strict;
use warnings;
use JSON::MaybeXS qw(encode_json decode_json);


#variable declarations
my %data;
my $username;

#load data from file if it exists
if (-e "coasterdata.json") {
    open DATA, '<', "./coasterdata.json"
    or die "could not open coasterdata.json $!";
    my $jsondata = (<DATA>);
    close DATA;
#decode json string and populate data hash    
    my $jsondecoded = decode_json $jsondata;
    %data = %$jsondecoded;
    $username = $data{username};
    print "\nWelcome back to Coaster Counter, $username!\n\n"
} else {
#prompt for username if no data exists
    print "\nWelcome to Coaster Counter!\n";
    do {
        print "\nBefore we begin, please enter your name: ";
        $username = <STDIN>;
        chomp($username);
    } while ((not defined $username) || ($username eq ''));
    $data{username} = $username;
    print "Thank you, $username!\n\n";
}

coastercounter();                                       #coastercounter subroutine

#variable dclarations
my $userselection = '';
my $parkname;
my $coastername;
my $useryn;

#main loop - prompt for user selection
while ($userselection ne "x" && $userselection ne "s"){
    print "Would you like to (a)dd a coaster, (l)ist the coasters, (s)ave and exit, or e(x)it without saving?\n";
    print "Please select (a/l/s/x): ";
    $userselection = <STDIN>;
    chomp($userselection);
    $userselection = lc $userselection;
    if ($userselection eq "a") {                        #add coaster data
        do{
            print "Please enter the name of the park where the coaster is located: ";
            $parkname = <STDIN>;
            chomp ($parkname);            
        } while ((not defined $parkname) || ($parkname eq ''));
        do{
            print "Please enter the name of the coaster from $parkname: ";
            $coastername = <STDIN>;
            chomp ($coastername);
        }while ((not defined $coastername) || ($coastername eq ''));
        do{
            print "Would you like to add $coastername from $parkname to your coaster count? (y/n): ";
            $useryn = <STDIN>;
            chomp ($useryn);
            $useryn = lc $useryn;
            if ($useryn eq "y"){
                if (defined($data{$parkname})) {
                    push (@{$data{$parkname}}, $coastername);
                } else {
                    $data{$parkname}[0] = $coastername;
                }
                print "$coastername from $parkname added!\n\n"
            } elsif ($useryn eq "n"){
                print "Coaster not added\n\n";
            }
        } while (($useryn ne "y") && ($useryn ne "n"));
        
    } elsif ($userselection eq "l") {                     #list coaster data
        print "\nThese are the coasters you've experienced:\n";
        for my $park (sort keys %data) {
            if ($park ne "username") {
                print "\nAt $park:\n";
                for my $i (0 .. $#{$data{$park}}) {
                    print "\t\t" . ($i + 1) . ".  $data{$park}[$i]\n"
                }
            }
        print "\n";
        }
        coastercounter();                                #coastercounter subroutine
    } elsif ($userselection eq "s") {                    #save data to json file
        open DATA, '>', "./coasterdata.json"
        or die "Could not open coasterdata.json: $!";
        my $jsondata = encode_json(\%data);
        print DATA $jsondata;
        close DATA;
        print "\nThanks for using Coaster Counter!\n";
        print "Saving and exiting...\nHave a great day!";
        exit;
    } elsif ($userselection eq "x") {                   #exit program without saving
        print "\nThanks for using Coaster Counter!\n";
        print "Exiting without saving...\nHave a great day!";
        exit;
    } else {
        print "Invalid selection\n";
    }
}

sub coastercounter {
    #Variable declarations
    my $coastercount = 0;
    my $milestone = '';
    my $togo = '';
    
    #Total items in all arrays for coaster count
    for my $park (keys %data){
        if ($park ne "username"){
            for my $i (0 .. $#{$data{$park}}) {
                $coastercount++;
            }
        }
    }
    
    #Determine next milestone and difference (milestone - coastercount)
    if ($coastercount < 50){
        $milestone = 50;
        $togo = 50 - $coastercount;
    } elsif ($coastercount < 100){
        $milestone = 100;
        $togo = 100 - $coastercount;
    } elsif ($coastercount < 200){
        $milestone = 200;
        $togo = 200 - $coastercount;
    } elsif ($coastercount < 300){
        $milestone = 300;
        $togo = 300 - $coastercount;
    } elsif ($coastercount < 400){
        $milestone = 400;
        $togo = 400 - $coastercount;
    } elsif ($coastercount < 500){
        $milestone = 500;
        $togo = 500 - $coastercount;
    } elsif ($coastercount < 1000){
        $milestone = 1000;
        $togo = 1000 - $coastercount;
    } else {
        $milestone = 4639;
        $togo = 4639 - $coastercount;
    }
    
    #Display coaster count and coasters remaining to next milestone
    print "It looks like you've experienced $coastercount rollercoasters so far.\n";
    if ($coastercount < 1000){
        print "Only $togo coasters to go until you reach $milestone!\n\n";
    } else {
        print "Wow! You've been on over 1000 rollercoasters!\n";
        print "There are $milestone operating rollercoasters in the world (as of 2018)... just $togo to go!\n\n";
    }

}